import com.haxepunk.HXP;
import com.haxepunk.Entity;
import com.haxepunk.graphics.*;
import flash.geom.Point;
import Units;

/** three types of damage and armour */
class DamageType {
	public var kinetic: Float;
	public var heat: Float;
	public var electric: Float;
	public function new(k:Float, h:Float, e:Float) {
		kinetic=k; heat=h; electric=e;
	}
}


/** Building base class */
class Building extends Entity {
	
	public var health: Float;
	public var level: Int = 1;
	public var info: String;

	public var built: Float;
	public var buildTime: Float;
	public var resourceCost: Float;
	public var energyCost: Float;
	public var maxHealth: Float;
	public var armour: DamageType;

	public var resources: Float;
	public var energy: Float;
	public var enabled: Bool;

	public var links: Array<Building>;
	public var inputEnergy: Float;
	public var inputResource: Float;

	public var rotating: Array<Image>;

	public function new(x:Float, y:Float) {
		super(x, y);
	}
	public override function added() {
		health = maxHealth;
		built = 0;
		setHitboxTo(graphic);
		if(width==0) setHitboxTo( cast(graphic, Graphiclist).children[0] );
	}

	public function getResourceDemand() {
		if(!enabled) return 0.0;
		if(built<buildTime) return resourceCost / buildTime;
		if(resources<0) return resources;
		return 0.0;
	}
	public function getEnergyDemand() {
		if(!enabled) return 0.0;
		if(built<buildTime) return -energyCost / buildTime;
		if(energy<0) return energy;
		return 0.0;
	}
	public function upgrade() {
		if(level==0) return; // cant upgrade
		var h:Float = health / maxHealth;
		built = buildTime;
		buildTime *= 1.8;
		energyCost *= 1.4;
		resourceCost *= 1.5;
		maxHealth *= 1.2;
		health = maxHealth * h;
		resources *= resources<0? 1.3: 1.1;
		energy *= energy<0? 1.5: 1.1;
		++level;
	}
	public function getEfficiency() {
		var rd:Float = getResourceDemand();
		var ed:Float = getEnergyDemand();
		var e: Float = 1.0;
		if(rd>0 && ed>0) e = Math.min(inputResource/rd, inputEnergy/ed);
		else if(rd>0) e = inputResource / rd;
		else if(ed>0) e = inputEnergy / ed;
		return Math.min(e, 1.0); // Clamp at 1.0
	}

	public function updateLinks() {
		if(links==null) return; // no links
		var le:Float=0, lr:Float=0;
		for(l in links) {
			le += l.getEnergyDemand();
			lr += l.getResourceDemand();
		}
		if(le==0) le=1; // avoid div0. shouldnt make a difference as demand will be 0
		if(lr==0) lr=1;
		for(l in links) {
			l.inputEnergy = l.getEnergyDemand() / le * inputEnergy;
			l.inputResource = l.getEnergyDemand() / lr * inputResource;
		}
	}

	public override function update() {
		updateLinks();
		var e: Float = getEfficiency();
		if(built<buildTime) built += e;
		if(health<maxHealth) health += e * 0.1; // repair
		if(rotating!=null) for(r in rotating) r.angle += e * 4;
	}

	public function damage(d:DamageType) {
		health -= d.kinetic  / armour.kinetic * 10;
		health -= d.heat     / armour.heat * 10;
		health -= d.electric / armour.electric * 10;
		if(health<=0) {
			world.remove(this);
			cast(world,Level).grid.remove(this);
		}
	}

	public function addImage(img:String, px:Int=0, py:Int=0, r:Bool=false) {
		var i:Image = new Image(img);
		i.centerOrigin();
		i.x += px;
		i.y += py;
		addGraphic(i);
		if(r) {
			if(rotating==null) rotating = [i];
			else rotating.push(i);
		}
		return i;
	}
	public function addAnimation(img:String, px:Int, py:Int, c:Int, f:Array<Int>, s:Float) {
		var w:Int =Std.int( HXP.getBitmap(img).width / c);
		var h:Int = HXP.getBitmap(img).height;
		var a:Spritemap = new Spritemap(img, w, h);
		a.centerOrigin();
		a.playFrames(f, s);
		addGraphic(a);
		return a;
	}
}


// ======================= Building types here ========================== //
class Generator extends Building {
	public function new(x:Float, y:Float) {
		super(x, y);
		info = "Stuff needs power";
		buildTime    = 100;
		resourceCost = 0;
		energyCost   = 0;
		maxHealth    = 30;
		armour       = new DamageType(10, 8, 3);
		resources    = 0;
		energy       = 10;
		level        = 1;
		addAnimation("graphics/interlink.png", 0,0, 2, [0,1], 3);
		links = [];
	}
}
class PowerNode extends Building {
	public function new(x:Float, y:Float) {
		super(x, y);
		info = "Extend power lines with this handy node";
		buildTime    = 10;
		resourceCost = 1;
		energyCost   = 5;
		maxHealth    = 10;
		armour       = new DamageType(10, 8, 3);
		resources    = 0;
		energy       = 0;
		level        = 0;
		addImage("graphics/node.png");
		links = [];
	}
}

class Extractor extends Building {
	public function new(x:Float, y:Float) {
		super(x, y);
		info = "Extractors mine valuable resources out of the ground.";
		buildTime    = 80;
		resourceCost = 0;
		energyCost   = 10;
		maxHealth    = 10;
		armour       = new DamageType(10, 8, 3);
		resources    = 5;
		energy       = -2;
	//	addAnimation("graphics/extractor2.png", 0,0, 3, [0,1,2,1], 5);
		addImage("graphics/extractor.png");
		addImage("graphics/extractor-r.png", 0, 0, true);
	}
}
class Radar extends Building {
	public function new(x:Float, y:Float) {
		super(x, y);
		info = "Advanced enemy detection";
		buildTime    = 90;
		resourceCost = 40;
		energyCost   = 30;
		maxHealth    = 5;
		armour       = new DamageType(7, 5, 1);
		resources    = 0;
		energy       = -10;
		addImage("graphics/radar.png", 0, 0, true);
	}
}

class TurretBuilding extends Building {
	var target:Entity;
	var direction: Point = HXP.zero;
	var fire:Bool = false;
	var angle:Float = 0;
	var turrets: Array<Turret> = [];
	var range: Float = 250;
	public function new(x:Float, y:Float) {
		super(x, y);
		info = "Something to shoot back";
		buildTime    = 20;
		resourceCost = 20;
		energyCost   = 10;
		maxHealth    = 10;
		armour       = new DamageType(10, 8, 3);
		resources    = 0;
		energy       = -1;
		turrets.push( new Turret(this, 0, 0, "graphics/turret.png", true) );
		setHitbox(10,10,5,5);
	}
	public override function added() {
		for(t in turrets) world.add(t);
	}
	public override function removed() {
		for(t in turrets) world.remove(t);
	}
	public function selectTarget() {
		var e:Array<Entity> = [];
		world.getType("unit", e);
		if(e.length==0) target = null;
		else {
			// closest target
			var best:Float=range*range*2.25;
			for(t in e) {
				var d:Float = HXP.distanceSquared(x,y,t.x,t.y);
				if(d<best) { best=d; target=t; }
			}
		}
		fire = false;
	}
	public override function update() {
		if(type=="") return;
		if(target==null) selectTarget();
		else if(cast(target,Unit).health<=0) target = null;
		else fire = HXP.distanceSquared(x,y,target.x, target.y)<range*range;
	}
}

