import com.haxepunk.HXP;
import com.haxepunk.Entity;
import com.haxepunk.utils.Input;
import com.haxepunk.utils.Draw;
import com.haxepunk.graphics.Text;

/** Menu of buildings you can build */
class Menu extends Entity {
	public var items:Array<Building> = [];
	public var selected: Int = -1;
	public var selectedBuilding: Building;
	public var over: Int = -1;
	private var size: Float = 40;
	public var tooltip: Text;

	public function new() {
		super(0, 50);
		setHitbox(40, 0);
		tooltip = new Text("");
		tooltip.color = 0x00;
		tooltip.alpha = 0.5;
	}
	public override function added() {
		world.addGraphic(tooltip, -1, 0, 0);
	}

	public function addBuilding(b:Building) {
		items.push(b);
		b.x = x + size/2;
		b.y = bottom + size/2;
		height = Std.int(items.length * size);
	}

	public override function update() {

		// Placement mode
		if(selectedBuilding!=null) {
			selectedBuilding.x = world.mouseX;
			selectedBuilding.y = world.mouseY;
			if(Input.mousePressed && canPlaceBuilding()) placeBuilding();
		}

		// Select building
		if(collidePoint(x, y, world.mouseX, world.mouseY)) {
			over = Std.int((world.mouseY - y) / size);
			if(Input.mouseDown) {
				createBuilding(over);
			}
		} else over = -1;

		if(over!=-1) {
			tooltip.x = world.mouseX + 10;
			tooltip.y = world.mouseY - 10;
			tooltip.text = items[over].info;
		} else tooltip.text="";
	}

	public override function render() {
		Draw.rectPlus(x-5, y, width+5, height, 0x202020, 0.4, false, 3);
		for(b in items) b.render();
		if(selected>=0) {
			Draw.rectPlus(x+3, y+selected*size+5, width-10, width-10, 0xff9010, 0.8, false, 3);
		}

		// Placement
		if(canPlaceBuilding()) {
			var list:Array<Building> = [];
			getLinks(world.mouseX, world.mouseY, 60, list);
			for(b in list) Draw.linePlus(Std.int(b.x), Std.int(b.y), world.mouseX, world.mouseY, 0x00, 0.5, 1.5);
		}
	}

	public function createBuilding(s:Int) {
		if(selected==s) return;
		selected = s;
		if(selectedBuilding!=null) world.remove(selectedBuilding);
		selectedBuilding = Type.createInstance( Type.getClass( items[selected] ), [0,0] );
		world.add(selectedBuilding);
	}
	public function placeBuilding() {
		// Add links
		var list:Array<Building> = [];
		getLinks(world.mouseX, world.mouseY, 60, list);
		for(b in list) cast(world, Level).grid.addLink(selectedBuilding, b);
		// clear
		selectedBuilding.type = "building";
		selectedBuilding.built = 0;
		selectedBuilding.health = selectedBuilding.maxHealth;
		selectedBuilding = null;
		selected = -1;
	}
	public function deselect() {
		if(selectedBuilding!=null) world.remove(selectedBuilding);
		selectedBuilding = null;
		selected = -1;
	}

	public function canPlaceBuilding() {
		if(selectedBuilding==null) return false;
		if(collideWith(selectedBuilding, x, y)!=null) return false;
		return selectedBuilding.collideTypes(["building", "unit"], selectedBuilding.x, selectedBuilding.y)==null;
	}

	public function getLinks(x:Float, y:Float, max:Float, out:Array<Building>) {
		// consumer - just nearest power supply
		// power - nearest power supply plus all unlinked consumers


		if(selectedBuilding.links==null /*|| selectedBuilding.energy<=0*/) {
			var b:Building = nearestNode(world.mouseX, world.mouseY, 60);
			if(b!=null) out.push(b);
		} else {
			allNodes(world.mouseX, world.mouseY, 60, out);
		}
	}

	public function nearestNode(x:Float, y:Float, max:Float) {
		var a:Array<Entity> = [];
		world.getType("building", a);
		if(a.length==0) return null;
		max *= max;
		var out:Building = null;
		for(b in a) {
			if(cast(b, Building).links!=null) {
				var d:Float = HXP.distanceSquared(x, y, b.x, b.y);
				if(d<max) { max=d; out = cast b; }
			}
		}
		return out;
	}
	public function allNodes(x:Float, y:Float, max:Float, out:Array<Building>) {
		var a:Array<Entity> = [];
		world.getType("building", a);
		if(a.length==0) return 0;
		max *= max;
		for(b in a) {
			if(HXP.distanceSquared(x, y, b.x, b.y)<max) {
				if(cast(b,Building).links!=null || !cast(world,Level).grid.isLinked(cast b))
					out.push( cast b);
			}
		}
		return out.length;
	}



}
