import com.haxepunk.HXP;
import com.haxepunk.Entity;
import com.haxepunk.graphics.Image;
import flash.geom.Point;
import Building;
import Units;

/** Go with an unoptimised version for simplicity */
class Bullet extends Entity {
	public var vx:Float;
	public var vy:Float;
	public var life: Int;
	public var damage: DamageType;
	public var speed: Float = 10;
	public var player: Bool;

	public function new(x:Float, y:Float, dx:Float, dy:Float, player:Bool) {
		super(x, y);
		this.vx = dx * speed;
		this.vy = dy * speed;
		this.player = player;
		life = 60;
		graphic = new Image("graphics/bullet.png");
		damage = new DamageType(3, 1, 0);
	}
	public override function added() {
		if(graphic!=null) cast(graphic, Image).angle = Math.atan2(vy, vx) * HXP.DEG - 90;
	}

	public override function update() {
		if(--life<0) world.remove(this);
		else {
			x += vx;
			y += vy;
			if(player) {
				var e:Unit = cast world.collidePoint("unit", x, y);
				if(e!=null) {
					life = 0;
					e.damage( damage );
					return;
				}
			} else {
				var b:Building = cast world.collidePoint("building", x, y);
				if(b!=null) {
					life = 0;
					b.damage( damage );
					return;
				}
				// Hit shield
				// ..?
			}
		}
	}
}

/** Does this work? */
abstract TurretBase ({x:Float, y:Float, direction:Point, angle:Float, fire:Bool, target:Entity}) {
	private function new(obj:Dynamic) this = obj;

	public var x(get, never): Float;
	private inline function get_x(): Float return this.x;

	public var y(get, never): Float;
	private inline function get_y(): Float return this.y;

	public var direction(get, never): Point;
	private inline function get_direction(): Point return this.direction;

	public var angle(get, never): Float;
	private inline function get_angle(): Float return this.angle;

	public var fire(get, never): Bool;
	private inline function get_fire(): Bool return this.fire;

	public var target(get, never): Entity;
	private inline function get_target(): Entity return this.target;

	@:from public static inline function fromUnit(u:Unit) return new TurretBase(u);
	@:from public static inline function fromBuilding(u:TurretBuilding) return new TurretBase(u);
}

class Turret extends Entity {
	public var parent: TurretBase;
	public var localX: Float;
	public var localY: Float;
	public var angle: Float = 0;
	public var image: Image;
	public var rotates: Bool;
	public var reloadTime: Int = 20;
	public var reload: Int = 0;
	public var rotation: Float = 0.1;
	public var player:Bool;

	public function new(parent:TurretBase, x:Float, y:Float, img:String=null, rot:Bool=false) {
		super(parent.x, parent.y);
		this.parent = parent;
		player = parent.direction.length==0;
		localX = x;
		localY = y;
		if(img!=null) {
			image = new Image(img);
			image.centerOrigin();
			graphic = image;
		}
		rotates = rot;
	}

	public override function update() {
		// Update position
		x = parent.x + parent.direction.x*localY + parent.direction.y * localX;
		y = parent.y + parent.direction.y*localY - parent.direction.x * localX;

		// Fire
		if(parent.fire && reload<=0) {
			var dx:Float = Math.cos(angle+parent.angle);
			var dy:Float = Math.sin(angle+parent.angle);
			world.add(new Bullet(x, y, dx, dy, player));
			reload = reloadTime;
		} else --reload;

		// Rotate
		if(rotates && parent.target!=null) {
			var ang:Float = Math.atan2(parent.target.y-y, parent.target.x-x);
			var diff: Float = ang - (angle+parent.angle);
			while(diff<-Math.PI) diff += Math.PI*2;
			while(diff>Math.PI) diff -= Math.PI*2;
			angle += HXP.clamp( diff, -rotation, rotation);
		}
		// Image
		if(graphic!=null) {
			cast(graphic, Image).angle = (angle + parent.angle) * HXP.DEG - 90;
		}
	}
}

