import com.haxepunk.HXP;
import com.haxepunk.Entity;
import com.haxepunk.graphics.*;
import flash.geom.Point;
import Building;

class Unit extends Entity {
	public var maxHealth: Float;
	public var health: Float;
	public var armour: DamageType;
	public var topSpeed: Float;
	public var rotation: Float;
	
	public var wreck: String;
	public var trail: String;

	public var direction: Point = new Point(0,1);
	public var target: Building;
	public var speed: Float;
	public var acceleration: Float;
	public var angle: Float;

	public var splat:String = null;
	public var track:String = null;
	public var image:Image;

	public var turrets:Array<Turret> = [];

	public var fire: Bool = false;

	public function new(x:Float, y:Float) {
		super(x, y);
		type = "unit";
	}

	public override function added() {
		setHitboxTo(graphic);
		health = maxHealth;
		speed = 0;
		angle = 0;
		for(t in turrets) world.add(t);
	}
	public override function removed() {
		for(t in turrets) world.remove(t);
	}

	public override function update() {
		super.update();

		if(target==null) selectTarget();
		else if(target.health<=0) target = null;
		else {
			var ang:Float = Math.atan2(target.y-y, target.x-x);
			var diff: Float = ang - angle;
			while(diff<-Math.PI) diff += Math.PI*2;
			while(diff>Math.PI) diff -= Math.PI*2;
			angle += HXP.clamp( diff, -rotation, rotation);
			direction.x = Math.cos(angle);
			direction.y = Math.sin(angle);

			// speed
			if(Math.abs(diff)>1) speed -= acceleration;
			else {
				var d:Float = HXP.distance(x, y, target.x, target.y);
				if(d>100) speed += acceleration;
				else speed -= acceleration;
			}
			speed = HXP.clamp(speed, 0, topSpeed);
			if(image!=null) image.angle = angle * HXP.DEG - 90;

			// Move
			if(speed>0) moveBy(direction.x*speed, direction.y*speed, ["building", "unit"]);

			// Shoot
			fire = Math.abs(diff)<0.1 && HXP.distanceSquared(x,y,target.x, target.y)<200*200;
		}
	}

	public function selectTarget() {
		// random target for now
		fire = false;
		var e:Array<Entity> = [];
		world.getType("building", e);
		if(e.length==0) target = null;
		else target = cast e[ HXP.rand(e.length) ];
	}

	public function addImage(img:String, px:Int=0, py:Int=0) {
		var i:Image = new Image(img);
		i.centerOrigin();
		i.x += px;
		i.y += py;
		addGraphic(i);
		return i;
	}
	public function addAnimation(img:String, px:Int, py:Int, c:Int, f:Array<Int>, s:Float) {
		var w:Int =Std.int( HXP.getBitmap(img).width / c);
		var h:Int = HXP.getBitmap(img).height;
		var a:Spritemap = new Spritemap(img, w, h);
		a.centerOrigin();
		a.playFrames(f, s);
		addGraphic(a);
		return a;
	}

	public function damage(d:DamageType) {
		health -= d.kinetic  / armour.kinetic * 10;
		health -= d.heat     / armour.heat * 10;
		health -= d.electric / armour.electric * 10;
		if(health<=0)  {
			world.remove(this);
			cast(world,Level).back.splat(this);
		}
	}
}



// ======================================================== //

class Tank extends Unit {
	public function new(x:Float, y:Float) {
		super(x, y);
		maxHealth = 10;
		armour = new DamageType(10, 8, 3);
		topSpeed = 2;
		acceleration = 0.1;
		rotation = 0.1;
		image = addImage("graphics/tank.png");
		turrets.push( new Turret(this, 0, -3, "graphics/tank-t.png", true));
		splat = "graphics/tank-x.png";
	}
}

